#!/usr/bin/env python

"""
Generate virtual keyboard image and layout for the osso-xterm-tkbd.
"""

import cairo, string

( CAIRO_FONT_WEIGHT_NORMAL,
  CAIRO_FONT_WEIGHT_BOLD ) = range( 2 )

( CAIRO_FONT_SLANT_NORMAL, 
  CAIRO_FONT_SLANT_ITALIC ) = ( 0, 2 )

M_PI = 3.14169

def DrawText(aContext, sText, nX0, nY0, nX1, nY1):
   nWidth = nX1 - nX0
   nHeight = nY1 - nY0

   bSmallEnough = False
   nFontSize = 40
   while not bSmallEnough:
      aContext.set_font_size(nFontSize)
      fascent, fdescent, fheight, fxadvance, fyadvance = aContext.font_extents()
      xbearing, ybearing, width, height, xadvance, yadvance = aContext.text_extents(sText)
      tx = nX0 - xbearing - width/2 + nWidth/2
      ty = nY0 - fdescent + fheight/2 + nHeight/2
      if tx < nX0 or nX1 < tx or ty < nY0 or nY1 < ty:
         nFontSize = nFontSize - 5
      else:
         bSmallEnough = True

   aContext.move_to(tx, ty)
   aContext.show_text(sText)

def DrawTextWithShadow(aContext, sText, nX0, nY0, nX1, nY1, lColor):
   fColorOffset = 0.3
   nShadowOffset = 1

   #lDarkerColor = [fComponent - fColorOffset for fComponent in lColor]
   lDarkerColor = (0., 0., 0., 0.2)

   aContext.set_source_rgba(*lDarkerColor)
   DrawText(aContext, sText, nX0, nY0 - nShadowOffset, nX1, nY1 - nShadowOffset)
   DrawText(aContext, sText, nX0 - nShadowOffset, nY0, nX1 - nShadowOffset, nY1)
   DrawText(aContext, sText, nX0, nY0 + nShadowOffset, nX1, nY1 + nShadowOffset)
   DrawText(aContext, sText, nX0 + nShadowOffset, nY0, nX1 + nShadowOffset, nY1)

   aContext.set_source_rgba(*lColor)
   DrawText(aContext, sText, nX0, nY0, nX1, nY1)
   

def DrawRoundedRect(aContext, nX0, nY0, nX1, nY1): 
   nWidth = nX1 - nX0
   nHeight = nY1 - nY0
 
   nRoundCorner = 10

   aContext.move_to(nX0 + nRoundCorner, nY0)
   aContext.line_to(nX1 - nRoundCorner, nY0)
   #it seems PI/2 is at 
   aContext.arc(nX1 - nRoundCorner, nY0 + nRoundCorner, nRoundCorner, 3*M_PI/2, 0)

   #aContext.move_to(nX1, nY0 + nRoundCorner)
   aContext.line_to(nX1, nY1 - nRoundCorner)
   aContext.arc(nX1 - nRoundCorner, nY1 - nRoundCorner, nRoundCorner, 0, M_PI/2)

   #aContext.move_to(nX1 - nRoundCorner, nY1)
   aContext.line_to(nX0 + nRoundCorner, nY1)
   aContext.arc(nX0 + nRoundCorner, nY1 - nRoundCorner, nRoundCorner, M_PI/2, M_PI)

   #aContext.move_to(nX0, nY1 - nRoundCorner)
   aContext.line_to(nX0, nY0 + nRoundCorner)
   aContext.arc(nX0 + nRoundCorner, nY0 + nRoundCorner, nRoundCorner, M_PI, 3*M_PI/2)

   aContext.fill()
   #aContext.stroke()

class Rectangle:
   def __init__(self, x0, y0, x1, y1):
      self.nX0 = x0
      self.nY0 = y0
      self.nX1 = x1
      self.nY1 = y1

   def Width(self):
      return self.nX1 - self.nX0

   def Height(self):
      return self.nY1 - self.nY0
 
   def __repr__(self):
      return "%d %d %d %d" % (self.nX0, self.nY0, self.nX1, self.nY1)

   def __str__(self):
      return repr(self)

class Color(list):
   def __init__(self, r, g, b, a):
      list.__init__(self, [r, g, b, a])

   def __repr__(self):
      return '%3.2f %3.2f %3.2f %3.2f' % tuple(self)

   def __str__(self):
      return repr(self)

class KeyInfo:
   def __init__(self, keyval, symbol, keyval_shift, symbol_shift, x0, y0, x1, y1, fgColor, bgColor):
      self.sKeyval = keyval
      self.sSymbol = symbol
      self.sKeyval_shifted = keyval_shift
      self.sSymbol_shifted = symbol_shift
      self.aCoord = Rectangle(x0, y0, x1, y1)
      self.lFgColor = Color(*fgColor)
      self.lBgColor = Color(*bgColor)


   def Draw(self, aContext):
      nKeyClearBorder = 4

      aContext.set_source_rgba(*self.lBgColor)
      aContext.set_line_width(1)

      #aContext.rectangle(self.nX0 + nKeyClearBorder, self.nY0 + nKeyClearBorder, 
      #                   self.Width() - nKeyClearBorder, self.Height() - nKeyClearBorder)
      DrawRoundedRect(aContext, self.aCoord.nX0 + nKeyClearBorder, self.aCoord.nY0 + nKeyClearBorder, 
                      self.aCoord.nX1 - nKeyClearBorder, self.aCoord.nY1 - nKeyClearBorder) 

      aContext.set_source_rgba(*self.lFgColor)

      aContext.select_font_face("Monospace", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD)

      if self.sSymbol.upper() == self.sSymbol_shifted.upper():
         DrawTextWithShadow(aContext, self.sSymbol, 
                  self.aCoord.nX0 + nKeyClearBorder, self.aCoord.nY0 + nKeyClearBorder, 
                  self.aCoord.nX1 - nKeyClearBorder, self.aCoord.nY1 - nKeyClearBorder,
                  self.lFgColor)
      else:
         DrawTextWithShadow(aContext, self.sSymbol, 
                  self.aCoord.nX0 + nKeyClearBorder, self.aCoord.nY0 + self.aCoord.Height()/2 + nKeyClearBorder, 
                  self.aCoord.nX1 - nKeyClearBorder, self.aCoord.nY1 - nKeyClearBorder, self.lFgColor) 
         DrawTextWithShadow(aContext, self.sSymbol_shifted, 
                  self.aCoord.nX0 + nKeyClearBorder, self.aCoord.nY0 + nKeyClearBorder, 
                  self.aCoord.nX1 - nKeyClearBorder, self.aCoord.nY0 + self.aCoord.Height()/2 - nKeyClearBorder,
                  self.lFgColor)

   def __repr__(self):
      sStr = '%s %s %s %s %s %s %s' %(self.sKeyval, self.sSymbol, 
                                      self.sKeyval_shifted, self.sSymbol_shifted, 
                                      self.aCoord, self.lFgColor, self.lBgColor)
      return sStr

   def __str__(self):
      return repr(self)

class Keyboard:
   def __init__(self, keys):
      self.lKeys = keys

   def Draw(self, aContext):
      for aKey in self.lKeys:
         aKey.Draw(aContext)
      
   def SaveToFile(self, sFileName):
      fileKeybLay = file(sFileName, "w")

      for aKey in self.lKeys:
         fileKeybLay.write('%s\n' % (aKey))

      fileKeybLay.close()

   def ReadLayoutFile(sLayoutFileName):
      lKeys = []

      fileKeybLay = file(sLayoutFileName, 'r')
      lLines = fileKeybLay.readlines()
      fileKeybLay.close()

      #for sLine in lLines:
      #   l

      return lKeys
 
def CreateStandardCompactLayout():
   arrKeybLayout = [ 
#digits row
[ ["1", "exclam", "1", "!"], ["2", "at", "2", "@"], ["3", "numbersign", "3", "#"], ["4", "dollar", "4", "$"], ["5", "percent", "5", "%"], ["6", "asciicircum", "6", "^"], 
  ["7", "ampersand", "7", "&"], ["8", "asterisk", "8", "*"], ["9", "parenleft", "9", "("], ["0", "parenright", "0", ")"], ["minus", "underscore", "-", "_"], ["equal", "plus", "=", "+"], ["BackSpace", "BackSpace", "<-", "<-"] 
],
#qwerty row  
[ ["Tab", "Tab", "Tab", "Tab"], ["q", "Q", "q", "Q"], ["w", "W", "w", "W"], ["e", "E", "e", "E"], ["r", "R", "r", "R"], ["t", "T", "t", "T"], ["y", "Y", "y", "Y"], ["u", "U","u", "U"], 
  ["i", "I", "i", "I"], ["o", "O", "o", "O"], ["p", "P", "p", "P"], ["bracketleft", "braceleft", "[", "{"], ["bracketright", "braceright", "]", "}"] 
],
#asdf row
[ ["Shift_L", "Shift_L", "Shift", "Shift"], ["a", "A", "a", "A"], ["s", "S", "s", "S"], ["d", "D", "d", "D"], ["f", "F", "f", "F"], ["g", "G", "g", "G"], ["h", "H", "h", "H"], ["j", "J", "j", "J"], 
  ["k", "K", "k", "K"], ["l", "L", "l", "L"], ["semicolon", "colon", ";", ":"], ["quoteright", "quotedbl", "'", "\""], ["KP_Enter", "KP_Enter", "Enter", "Enter"]
],
#zxcvb row
[ ["Ctrl_L", "Ctrl_L", "Ctrl", "Ctrl"], ["z", "Z", "z", "Z"], ["x", "X", "x", "X"], ["c", "C", "c", "C"], ["v", "V", "v", "V"], ["b", "B", "b", "B"], 
  ["n", "N", "n", "N"], ["m", "M", "m", "M"], 
  ["comma", "less", ",", "<"], ["period", "greater", ".", ">"], ["slash", "question", "/", "?"], ["backslash", "bar", "\\", "|"], 
  ["quoteleft", "asciitilde", "`", "~"], ["space", "space", "Spc", "Spc"] 
]
]; 

   def GetBgColor(sKey):
      if sKey in string.digits:
         return (0.2, 0.2, 0.8, 0.2)
      if sKey in string.letters:
         return (0.2, 0.8, 0.2, 0.2)
      if sKey in ("Shift", "Ctrl", "Tab", "Spc", "<-", "Enter"):
         return (0.8, 0.2, 0.2, 0.2)
      #if sKey in ("<-", "Enter"):
      #   return (0.2, 0.8, 0.8, 0.2)

      #default
      return (0.8, 0.8, 0.2, 0.2)

   #fileKeybLay = file(sLayoutFileName, "w")

   nMaxX = 780 #w/out the scrollbar
   nMaxY = 480
   #nMaxX = 2
   #nMaxY = 2
   nKeyClearBorder = 0
   lFgColor = (0.9, 0.9, 0.1, 0.3)
   lBgColor = (0.8, 0.8, 0.9, 0.2)
 
   nRows = len(arrKeybLayout)
   nKeyHeight = nMaxY / nRows

   lKeys = []
   for (nRow, lRow) in enumerate(arrKeybLayout):
      nCols = len(lRow)
      nKeyWidth = nMaxX / nCols
      for (nCol, lKeyInfo) in enumerate(lRow):
         #print lKeyInfo
         nX0 = int(nCol * nKeyWidth)
         nY0 = int(nRow * nKeyHeight)
         lBgColor = GetBgColor(lKeyInfo[2])
         aKey = KeyInfo(lKeyInfo[0], lKeyInfo[2], lKeyInfo[1], lKeyInfo[3], 
                        nX0, nY0, nX0 + nKeyWidth, nY0 + nKeyHeight, lFgColor, lBgColor)
         lKeys.append(aKey)
         #fileKeybLay.write('%s\n' % (aKey))

   #fileKeybLay.close()

   return lKeys


def GenerateImageFile(aKeyboard, sFileName):
   aSurface = cairo.ImageSurface(cairo.FORMAT_ARGB32, 800, 480)
   aContext = cairo.Context(aSurface)
 
   aKeyboard.Draw(aContext)

   aSurface.write_to_png(sFileName)

def main():
   lKeys = CreateStandardCompactLayout()
   theKeyboard = Keyboard(lKeys)
   theKeyboard.SaveToFile('tkbd.lay')
   GenerateImageFile(theKeyboard, 'tkbd.png')

main()

